@@ -13,7 +13,7 @@
 #include <string.h>
 #include <stdlib.h>
 
-#if defined(_MSC_VER) && _MSC_VER+0 <= 1800
+#if defined(_MSC_VER) && _MSC_VER + 0 <= 1800
 /* Substitution is OK while return value is not used */
 #define snprintf _snprintf
 #endif
@@ -33,7 +33,8 @@ struct connection_info_struct
   struct MHD_PostProcessor *postprocessor;
 };
 
-const char *askpage = "<html><body>\
+const char *askpage =
+  "<html><body>\
                        What's your name, Sir?<br>\
                        <form action=\"/namepost\" method=\"post\">\
                        <input name=\"name\" type=\"text\">\
@@ -47,17 +48,17 @@ const char *errorpage =
   "<html><body>This doesn't seem to be right.</body></html>";
 
 
-static int
+static enum MHD_Result
 send_page (struct MHD_Connection *connection, const char *page)
 {
-  int ret;
+  enum MHD_Result ret;
   struct MHD_Response *response;
 
 
   response =
     MHD_create_response_from_buffer (strlen (page), (void *) page,
-				     MHD_RESPMEM_PERSISTENT);
-  if (!response)
+                                     MHD_RESPMEM_PERSISTENT);
+  if (! response)
     return MHD_NO;
 
   ret = MHD_queue_response (connection, MHD_HTTP_OK, response);
@@ -67,135 +68,138 @@ send_page (struct MHD_Connection *connection, const char *page)
 }
 
 
-static int
+static enum MHD_Result
 iterate_post (void *coninfo_cls, enum MHD_ValueKind kind, const char *key,
               const char *filename, const char *content_type,
               const char *transfer_encoding, const char *data, uint64_t off,
               size_t size)
 {
   struct connection_info_struct *con_info = coninfo_cls;
-  (void)kind;               /* Unused. Silent compiler warning. */
-  (void)filename;           /* Unused. Silent compiler warning. */
-  (void)content_type;       /* Unused. Silent compiler warning. */
-  (void)transfer_encoding;  /* Unused. Silent compiler warning. */
-  (void)off;                /* Unused. Silent compiler warning. */
+  (void) kind;               /* Unused. Silent compiler warning. */
+  (void) filename;           /* Unused. Silent compiler warning. */
+  (void) content_type;       /* Unused. Silent compiler warning. */
+  (void) transfer_encoding;  /* Unused. Silent compiler warning. */
+  (void) off;                /* Unused. Silent compiler warning. */
 
   if (0 == strcmp (key, "name"))
+  {
+    if ((size > 0) && (size <= MAXNAMESIZE))
     {
-      if ((size > 0) && (size <= MAXNAMESIZE))
-        {
-          char *answerstring;
-          answerstring = malloc (MAXANSWERSIZE);
-          if (!answerstring)
-            return MHD_NO;
+      char *answerstring;
+      answerstring = malloc (MAXANSWERSIZE);
+      if (! answerstring)
+        return MHD_NO;
 
-          snprintf (answerstring, MAXANSWERSIZE, greetingpage, data);
-          con_info->answerstring = answerstring;
-        }
-      else
-        con_info->answerstring = NULL;
-
-      return MHD_NO;
+      snprintf (answerstring, MAXANSWERSIZE, greetingpage, data);
+      con_info->answerstring = answerstring;
     }
+    else
+      con_info->answerstring = NULL;
+
+    return MHD_NO;
+  }
 
   return MHD_YES;
 }
 
+
 static void
 request_completed (void *cls, struct MHD_Connection *connection,
                    void **con_cls, enum MHD_RequestTerminationCode toe)
 {
   struct connection_info_struct *con_info = *con_cls;
-  (void)cls;         /* Unused. Silent compiler warning. */
-  (void)connection;  /* Unused. Silent compiler warning. */
-  (void)toe;         /* Unused. Silent compiler warning. */
+  (void) cls;         /* Unused. Silent compiler warning. */
+  (void) connection;  /* Unused. Silent compiler warning. */
+  (void) toe;         /* Unused. Silent compiler warning. */
 
   if (NULL == con_info)
     return;
 
   if (con_info->connectiontype == POST)
-    {
-      MHD_destroy_post_processor (con_info->postprocessor);
-      if (con_info->answerstring)
-        free (con_info->answerstring);
-    }
+  {
+    MHD_destroy_post_processor (con_info->postprocessor);
+    if (con_info->answerstring)
+      free (con_info->answerstring);
+  }
 
   free (con_info);
   *con_cls = NULL;
 }
 
 
-static int
+static enum MHD_Result
 answer_to_connection (void *cls, struct MHD_Connection *connection,
                       const char *url, const char *method,
                       const char *version, const char *upload_data,
                       size_t *upload_data_size, void **con_cls)
 {
-  (void)cls;               /* Unused. Silent compiler warning. */
-  (void)url;               /* Unused. Silent compiler warning. */
-  (void)version;           /* Unused. Silent compiler warning. */
+  (void) cls;               /* Unused. Silent compiler warning. */
+  (void) url;               /* Unused. Silent compiler warning. */
+  (void) version;           /* Unused. Silent compiler warning. */
 
   if (NULL == *con_cls)
+  {
+    struct connection_info_struct *con_info;
+
+    con_info = malloc (sizeof (struct connection_info_struct));
+    if (NULL == con_info)
+      return MHD_NO;
+    con_info->answerstring = NULL;
+
+    if (0 == strcmp (method, "POST"))
     {
-      struct connection_info_struct *con_info;
+      con_info->postprocessor =
+        MHD_create_post_processor (connection, POSTBUFFERSIZE,
+                                   iterate_post, (void *) con_info);
 
-      con_info = malloc (sizeof (struct connection_info_struct));
-      if (NULL == con_info)
+      if (NULL == con_info->postprocessor)
+      {
+        free (con_info);
         return MHD_NO;
-      con_info->answerstring = NULL;
+      }
 
-      if (0 == strcmp (method, "POST"))
-        {
-          con_info->postprocessor =
-            MHD_create_post_processor (connection, POSTBUFFERSIZE,
-                                       iterate_post, (void *) con_info);
+      con_info->connectiontype = POST;
+    }
+    else
+      con_info->connectiontype = GET;
 
-          if (NULL == con_info->postprocessor)
-            {
-              free (con_info);
-              return MHD_NO;
-            }
+    *con_cls = (void *) con_info;
 
-          con_info->connectiontype = POST;
-        }
-      else
-        con_info->connectiontype = GET;
+    return MHD_YES;
+  }
 
-      *con_cls = (void *) con_info;
+  if (0 == strcmp (method, "GET"))
+  {
+    return send_page (connection, askpage);
+  }
+
+  if (0 == strcmp (method, "POST"))
+  {
+    struct connection_info_struct *con_info = *con_cls;
+
+    if (*upload_data_size != 0)
+    {
+      MHD_post_process (con_info->postprocessor, upload_data,
+                        *upload_data_size);
+      *upload_data_size = 0;
 
       return MHD_YES;
     }
-
-  if (0 == strcmp (method, "GET"))
-    {
-      return send_page (connection, askpage);
-    }
-
-  if (0 == strcmp (method, "POST"))
-    {
-      struct connection_info_struct *con_info = *con_cls;
-
-      if (*upload_data_size != 0)
-        {
-          MHD_post_process (con_info->postprocessor, upload_data,
-                            *upload_data_size);
-          *upload_data_size = 0;
-
-          return MHD_YES;
-        }
-      else if (NULL != con_info->answerstring)
-        return send_page (connection, con_info->answerstring);
-    }
+    else if (NULL != con_info->answerstring)
+      return send_page (connection, con_info->answerstring);
+  }
 
   return send_page (connection, errorpage);
 }
 
+
 int
 main ()
 {
   struct MHD_Daemon *daemon;
 
-  daemon = MHD_start_daemon (MHD_USE_AUTO | MHD_USE_INTERNAL_POLLING_THREAD, PORT, NULL, NULL,
+  daemon = MHD_start_daemon (MHD_USE_AUTO | MHD_USE_INTERNAL_POLLING_THREAD,
+                             PORT, NULL, NULL,
                              &answer_to_connection, NULL,
                              MHD_OPTION_NOTIFY_COMPLETED, request_completed,
                              NULL, MHD_OPTION_END);
