@@ -27,11 +27,11 @@
 #include "platform.h"
 #include <microhttpd.h>
 
-#if defined(CPU_COUNT) && (CPU_COUNT + 0) < 2
-#undef CPU_COUNT
+#if defined(MHD_CPU_COUNT) && (MHD_CPU_COUNT + 0) < 2
+#undef MHD_CPU_COUNT
 #endif
-#if ! defined(CPU_COUNT)
-#define CPU_COUNT 2
+#if ! defined(MHD_CPU_COUNT)
+#define MHD_CPU_COUNT 2
 #endif
 
 
@@ -44,9 +44,9 @@ ahc_echo (void *cls,
           const char *upload_data, size_t *upload_data_size,
           void **unused)
 {
-  (void) cls; (void) connection; (void) url;          /* Unused. Silent compiler warning. */
-  (void) method; (void) version; (void) upload_data;  /* Unused. Silent compiler warning. */
-  (void) upload_data_size; (void) unused;           /* Unused. Silent compiler warning. */
+  (void) cls; (void) connection; (void) url;         /* Unused. Silent compiler warning. */
+  (void) method; (void) version; (void) upload_data; /* Unused. Silent compiler warning. */
+  (void) upload_data_size; (void) unused;            /* Unused. Silent compiler warning. */
 
   return MHD_NO;
 }
@@ -92,7 +92,8 @@ testMultithreadedPoolGet (int poll_flag)
   d = MHD_start_daemon (MHD_USE_INTERNAL_POLLING_THREAD | MHD_USE_ERROR_LOG
                         | poll_flag,
                         0, NULL, NULL, &ahc_echo, "GET",
-                        MHD_OPTION_THREAD_POOL_SIZE, CPU_COUNT, MHD_OPTION_END);
+                        MHD_OPTION_THREAD_POOL_SIZE, MHD_CPU_COUNT,
+                        MHD_OPTION_END);
   if (d == NULL)
     return 4;
   MHD_stop_daemon (d);
