@@ -169,14 +169,14 @@ main (int argc, char *const *argv)
     curl_global_cleanup ();
     return 77;
   }
-  if (0 != strncmp (ssl_version, "GnuTLS", 6))
+  if (! curl_tls_is_gnutls ())
   {
     fprintf (stderr, "This test can be run only with libcurl-gnutls.\n");
     curl_global_cleanup ();
     return 77;
   }
 
-  if (curl_uses_nss_ssl () == 0)
+  if (curl_tls_is_nss ())
   {
     aes256_sha = "rsa_aes_256_sha";
   }
