@@ -1,6 +1,7 @@
 /*
      This file is part of libmicrohttpd
      Copyright (C) 2010, 2012 Christian Grothoff
+     Copyright (C) 2016-2021 Evgeny Grin (Karlson2k)
 
      libmicrohttpd is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
@@ -22,6 +23,7 @@
  * @file daemontest_digestauth_with_arguments.c
  * @brief  Testcase for libmicrohttpd Digest Auth with arguments
  * @author Amr Ali
+ * @author Karlson2k (Evgeny Grin)
  */
 #include "MHD_config.h"
 #include "platform.h"
@@ -85,7 +87,8 @@ ahc_echo (void *cls,
   const char *password = "testpass";
   const char *realm = "test@example.com";
   enum MHD_Result ret;
-  (void) cls; (void) url;                          /* Unused. Silent compiler warning. */
+  int ret_i;
+  (void) cls; (void) url;                         /* Unused. Silent compiler warning. */
   (void) method; (void) version; (void) upload_data; /* Unused. Silent compiler warning. */
   (void) upload_data_size; (void) unused;         /* Unused. Silent compiler warning. */
 
@@ -103,13 +106,13 @@ ahc_echo (void *cls,
     MHD_destroy_response (response);
     return ret;
   }
-  ret = MHD_digest_auth_check (connection, realm,
-                               username,
-                               password,
-                               300);
+  ret_i = MHD_digest_auth_check (connection, realm,
+                                 username,
+                                 password,
+                                 300);
   free (username);
-  if ( (ret == MHD_INVALID_NONCE) ||
-       (ret == MHD_NO) )
+  if ( (ret_i == MHD_INVALID_NONCE) ||
+       (ret_i == MHD_NO) )
   {
     response = MHD_create_response_from_buffer (strlen (DENIED),
                                                 DENIED,
@@ -119,8 +122,8 @@ ahc_echo (void *cls,
     ret = MHD_queue_auth_fail_response (connection, realm,
                                         MY_OPAQUE,
                                         response,
-                                        (ret == MHD_INVALID_NONCE) ? MHD_YES :
-                                        MHD_NO);
+                                        (ret_i == MHD_INVALID_NONCE) ?
+                                        MHD_YES : MHD_NO);
     MHD_destroy_response (response);
     return ret;
   }
@@ -193,7 +196,7 @@ testDigestAuth ()
                GetLastError ());
       return 1;
     }
-    b = CryptGenRandom (cc, 8, (BYTE*) rnd);
+    b = CryptGenRandom (cc, 8, (BYTE *) rnd);
     if (b == 0)
     {
       fprintf (stderr, "Failed to generate 8 random bytes: %lu\n",
@@ -278,5 +281,5 @@ main (int argc, char *const *argv)
   if (errorCount != 0)
     fprintf (stderr, "Error (code: %u)\n", errorCount);
   curl_global_cleanup ();
-  return errorCount != 0;       /* 0 == pass */
+  return (0 == errorCount) ? 0 : 1;       /* 0 == pass */
 }
